#! /bin/env python

import os.path
import subprocess
import sys

if __name__ == '__main__':
    python = sys.executable

    # create buildout.cfg
    print 'creating buildout.cfg ...'
    if os.path.exists('buildout.cfg'):
        print "ERROR: buildout.cfg already exists."
        print "       Please rename the existing one and restart install."
        sys.exit(-1)
    buildout_cfg = file('buildout.cfg', 'w')
    buildout_cfg.write('[buildout]\nextends = profiles/prod.cfg')
    buildout_cfg.close()

    # call boostrap.py
    print 'calling %s bootstrap.py ...' % python
    res = subprocess.call([sys.executable, 'bootstrap.py'])
    if res:
        sys.exit(res)

    # run buildout
    res = subprocess.call(['bin/buildout'])
    if res:
        sys.exit(res)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: addressbook.py 913 2009-01-02 12:47:38Z mac $

import zc.catalog.catalogindex
import zope.app.appsetup.bootstrap
import zope.app.catalog.catalog
import zope.app.catalog.interfaces
import zope.app.component.interfaces
import zope.app.component.site
import zope.app.container.btree
import zope.app.container.interfaces
import zope.app.intid
import zope.app.intid.interfaces
import zope.component
import zope.interface
import zope.location
import zope.schema.fieldproperty

import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.utils


class AddressBook(zope.app.container.btree.BTreeContainer, 
                  zope.app.component.site.SiteManagerContainer):
    "An address book."

    zope.interface.implements(icemac.addressbook.interfaces.IAddressBook)

    keywords = None

    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IAddressBook['title'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['notes'])

@zope.component.adapter(
    icemac.addressbook.interfaces.IAddressBook,
    zope.app.container.interfaces.IObjectAddedEvent)
def create_address_book_infrastructure(addressbook, event):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # add site manager
    if not zope.app.component.interfaces.ISite.providedBy(addressbook):
        site_manager = zope.app.component.site.LocalSiteManager(addressbook)
        addressbook.setSiteManager(site_manager)

    site_manager = addressbook.getSiteManager()

    # add keywords container
    if addressbook.keywords is None:
        keywords = icemac.addressbook.utils.create_obj(
            icemac.addressbook.keyword.KeywordContainer)
        addressbook.keywords = keywords
        zope.location.locate(keywords, addressbook, '++attribute++keywords')
        site_manager.registerUtility(
            keywords, icemac.addressbook.interfaces.IKeywords)

    add_more_addressbook_infrastructure(addressbook, addressbook)

@icemac.addressbook.utils.set_site
def add_more_addressbook_infrastructure(addressbook):
    "Add more infrastructure which depends on addressbook set as site."
    intids = zope.component.queryUtility(
        zope.app.intid.interfaces.IIntIds)
    if intids is None:
        # add intid utility
        intids = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.app.intid.interfaces.IIntIds, '',
            zope.app.intid.IntIds, asObject=True)

        # register persons with intid utility
        for person in addressbook.values():
            intids.register(person)
            # register adresses of persons
            for value in person.values():
                intids.register(value)

    catalog = zope.component.queryUtility(
        zope.app.catalog.interfaces.ICatalog)
    if catalog is None:
        # add catalog
        catalog = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.app.catalog.interfaces.ICatalog, '',
            zope.app.catalog.catalog.Catalog, asObject=True)

    # indexes
    if 'keywords' not in catalog:
        catalog['keywords'] = zc.catalog.catalogindex.SetIndex(
            'get_titles', icemac.addressbook.interfaces.IKeywords,
            field_callable=True)



import doctest
import os.path
import zope.app.testing.functional
# import zope.testing.doctest

import icemac.addressbook.address
import icemac.addressbook.addressbook
import icemac.addressbook.keyword
import icemac.addressbook.person
import icemac.addressbook.utils


ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
FunctionalLayer = zope.app.testing.functional.ZCMLLayer(
    ftesting_zcml, __name__, 'FunctionalLayer')


# def FunctionalDocTestSuite(module=None, **kw):
#     module = zope.testing.doctest._normalize_module(module)
#     suite = zope.app.testing.functional.FunctionalDocTestSuite(module, **kw)
#     suite.layer = FunctionalLayer
#     return suite


def FunctionalDocFileSuite(*paths, **kw):
    kw['optionflags'] = (kw.get('optionflags', 0) |
                         doctest.ELLIPSIS |
                         doctest.NORMALIZE_WHITESPACE)
    suite = zope.app.testing.functional.FunctionalDocFileSuite(*paths, **kw)
    suite.layer = FunctionalLayer
    return suite


class FunctionalTestCase(zope.app.testing.functional.FunctionalTestCase):
    layer = FunctionalLayer


def create_addressbook(parent=None, name='ab', title=u'test address book'):
    ab = icemac.addressbook.utils.create_obj(
        icemac.addressbook.addressbook.AddressBook, title=title)
    if parent is None:
        parent = zope.app.testing.functional.getRootFolder()
    parent[name] = ab
    return ab


def create_keyword(addressbook, title, notes=u''):
    parent = addressbook.keywords
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.keyword.Keyword, title=title, notes=notes)
    return parent[name]


@icemac.addressbook.utils.set_site
def create_person(parent, last_name, **kw):
    kw['last_name'] = last_name
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.person.Person, **kw)
    return parent[name]


@icemac.addressbook.utils.set_site
def create_postal_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PostalAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_postal_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_email_address(person, set_as_default=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.EMailAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_email_address = address
    return address


@icemac.addressbook.utils.set_site
def create_home_page_address(person, set_as_default=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.HomePageAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_home_page_address = address
    return address


@icemac.addressbook.utils.set_site
def create_phone_number(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PhoneNumber, **kw)
    number = person[name]
    if set_as_default:
        person.default_phone_number = number
    if return_obj:
        return number
